import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:nb_utils/nb_utils.dart';

extension WidgetExtension on Widget {
  SizedBox withSize({double width = 0.0, double height = 0.0}) => SizedBox(height: height, width: width, child: this);

  SizedBox withWidth(double width) => SizedBox(width: width, child: this);

  SizedBox withHeight(double height) => SizedBox(height: height, child: this);

  Padding paddingTop(double top) => Padding(padding: EdgeInsets.only(top: top), child: this);

  Padding paddingLeft(double left) => Padding(padding: EdgeInsets.only(left: left), child: this);

  Padding paddingRight(double right) => Padding(padding: EdgeInsets.only(right: right), child: this);

  Padding paddingBottom(double bottom) => Padding(padding: EdgeInsets.only(bottom: bottom), child: this);

  Padding paddingAll(double padding) => Padding(padding: EdgeInsets.all(padding), child: this);

  Padding paddingOnly({double top = 0.0, double left = 0.0, double bottom = 0.0, double right = 0.0}) {
    return Padding(
      padding: EdgeInsets.fromLTRB(left, top, right, bottom),
      child: this,
    );
  }

  Padding paddingSymmetric({double vertical = 0.0, double horizontal = 0.0}) {
    return Padding(
      padding: EdgeInsets.symmetric(vertical: vertical, horizontal: horizontal),
      child: this,
    );
  }

  Widget visible(bool visible, {Widget defaultWidget}) => visible ? this : defaultWidget ?? SizedBox();

  ClipRRect cornerRadiusWithClipRRectOnly({int bottomLeft = 0, int bottomRight = 0, int topLeft = 0, int topRight = 0}) {
    return ClipRRect(
      borderRadius: BorderRadius.only(
        bottomLeft: Radius.circular(bottomLeft.toDouble()),
        bottomRight: Radius.circular(bottomRight.toDouble()),
        topLeft: Radius.circular(topLeft.toDouble()),
        topRight: Radius.circular(topRight.toDouble()),
      ),
      child: this,
    );
  }

  ClipRRect cornerRadiusWithClipRRect(double radius) => ClipRRect(borderRadius: BorderRadius.all(Radius.circular(radius)), child: this);

  Visibility withVisibility(
    bool visible, {
    Widget replacement,
    bool maintainAnimation = false,
    bool maintainState = false,
    bool maintainSize = false,
    bool maintainSemantics = false,
    bool maintainInteractivity = false,
  }) {
    return Visibility(
      visible: visible,
      maintainAnimation: maintainAnimation,
      maintainInteractivity: maintainInteractivity,
      maintainSemantics: maintainSemantics,
      maintainSize: maintainSize,
      maintainState: maintainState,
      key: key,
      child: this,
      replacement: replacement ?? Container(),
    );
  }

  Widget opacity({@required double opacity, durationInSecond = 1}) {
    return AnimatedOpacity(opacity: opacity, duration: Duration(seconds: durationInSecond), child: this);
  }

  Widget rotate({@required double angle, bool transformHitTests = true, Offset origin}) {
    return Transform.rotate(origin: origin, angle: angle, transformHitTests: transformHitTests, child: this);
  }

  Widget center({double heightFactor, double widthFactor}) => Center(heightFactor: heightFactor, widthFactor: widthFactor, child: this);

  Container withRoundedCorners({
    Color backgroundColor = whiteColor,
    BorderRadius borderRadius = const BorderRadius.all(Radius.circular(8.0)),
    LinearGradient gradient,
    BoxBorder border,
    List<BoxShadow> boxShadow,
    DecorationImage decorationImage,
    BoxShape boxShape = BoxShape.rectangle,
  }) {
    return Container(
      decoration: boxDecorationWithRoundedCorners(
        backgroundColor: backgroundColor,
        borderRadius: borderRadius,
        gradient: gradient,
        border: border,
        boxShadow: boxShadow,
        decorationImage: decorationImage,
        boxShape: boxShape,
      ),
      child: this,
    );
  }

  Container withShadow({
    Color bgColor = whiteColor,
    Color shadowColor = Colors.black12,
    blurRadius = 10.0,
    spreadRadius = 0.0,
    Offset offset = const Offset(0.0, 0.0),
    LinearGradient gradient,
    BoxBorder border,
    DecorationImage decorationImage,
    BoxShape boxShape = BoxShape.rectangle,
  }) {
    return Container(
      decoration: boxDecorationWithShadow(
        boxShadow: [
          BoxShadow(
            color: shadowColor,
            blurRadius: blurRadius,
            spreadRadius: spreadRadius,
            offset: offset,
          ),
        ],
        backgroundColor: bgColor,
        gradient: gradient,
        border: border,
        decorationImage: decorationImage,
        boxShape: boxShape,
      ),
      child: this,
    );
  }

  Widget onTap(Function function, {BorderRadius borderRadius = const BorderRadius.all(Radius.circular(0))}) {
    return InkWell(
      onTap: function,
      borderRadius: borderRadius,
      child: this,
    );
  }

  Future<T> launch<T>(context, {bool isNewTask = false}) async {
    if (this is StatelessWidget || this is StatefulWidget) {
      if (isNewTask) {
        return await Navigator.of(context).pushAndRemoveUntil(MaterialPageRoute(builder: (context) => this), (route) => route == null);
      } else {
        return await Navigator.of(context).push(MaterialPageRoute<T>(builder: (_) => this));
      }
    } else {
      throw Exception('Error: Widget should be StatelessWidget or StatefulWidget');
    }
  }

  Widget withShaderMask(List<Color> colors, {BlendMode blendMode = BlendMode.srcATop}) {
    return withShaderMaskGradient(
      LinearGradient(colors: colors),
      blendMode: blendMode,
    );
  }

  Widget withShaderMaskGradient(Gradient gradient, {BlendMode blendMode = BlendMode.srcATop}) {
    return ShaderMask(
      shaderCallback: (rect) => gradient.createShader(rect),
      blendMode: blendMode,
      child: this,
    );
  }

  @deprecated
  Widget withScroll({
    ScrollPhysics physics,
    EdgeInsetsGeometry padding,
    Axis scrollDirection = Axis.vertical,
    ScrollController controller,
    DragStartBehavior dragStartBehavior = DragStartBehavior.start,
    bool primary,
    bool reverse,
  }) {
    return SingleChildScrollView(
      child: this,
      physics: physics,
      padding: padding,
      scrollDirection: scrollDirection,
      controller: controller,
      dragStartBehavior: dragStartBehavior,
      primary: primary,
      reverse: reverse,
      key: key,
    );
  }

  Widget expand({flex = 1}) => Expanded(child: this, flex: flex);

  Widget flexible({flex = 1, FlexFit fit}) => Flexible(child: this, flex: flex, fit: fit);
}
